
#ifndef __CALLBACKS_H__
#define __CALLBACKS_H__

#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <gtk/gtk.h>
#include <pango/pango.h>
#include <fcntl.h>
#include <glib/glib.h>
#include <dlfcn.h>
#include <postgresql/libpq-fe.h>
#include <pwd.h>
#include <unistd.h>
#include <errno.h>
#include <time.h>


/* Autorzy projektu XdeFactor-ng */
#define TITLE  "XdeFactor-ng v1.0"
#define ARMAD  "Michał Ociepka <armad@linuxindex.pl>"
#define MAZEK  "Marcin Mazurek <M.Mazurek@netsync.pl>"
#define ZNAHOR "Piotr Konieczny <znahor@boss.adcom.com.pl>"

/* Pliki konfiguracyjne */

/// Katalog domowy XdeFactor-ng
#define HOME_DIR "/etc/defactor-ng/x" 
/// Plik zawierajacy moduly ladowane automatycznie po starcie XdeFactor-ng
#define MODULES "/etc/defactor-ng/x/modules.conf"
/// Konfiguracja polaczenia z baza danych
#define DB	"/etc/defactor-ng/x/db.conf"
/// Plik z logami XdeFactor-ng
#define LOG	"/etc/defactor-ng/x/defactor-ng.log"

/* pozostale makra */
/// Wpis do logow przy starcie
#define XDEF_START  "XdeFactor-ng START"

/* Menu */
#define MENU_1 "Plik"
#define MENU_2 "Dodatki"
#define MENU_3 "Admin"
#define MENU_4 "Pomoc"

/* funkcje */



/**
  Funkcja do przeszukiwania wzorcow w plikach konfiguracyjnych.
  @param file_name    - plik do przeszukiwan
  @param model	      - wzorzec ktorego szukamy
  @param result       - tu wpisze sie rezultat przeszukiwania pliku

  Format danych w plikach:
  model=result

  @author Michal Ociepka <M.Ociepka@netsync.pl>
  @version 1.0
*/
void search (char *file_name, char *model, char *result);

/* moduly */
/*
  Ten modul bedzie ZAWSZE widoczny, bedzie pokazywal autorow
  calego projektu oraz logo. Ten modul jest wkompilowywany 
  standardowo i NIE DA SIE GO WYLACZYC!!
*/
void moduleAuthors ();

typedef struct 
{
  PGconn *conn;
  int connect; //dla wartosci 0 polaczenie OK, w przeciwnym wypadku -1
  int userID; //id uzytkownika, ktory sie zalogowal
  int kind_of_conn; //1 - polacz, 0 - rozlacz
  GString *ip;
} xdfConnModule;
/**
  Polaczenie z baza danych - PostgreSQL.
  Funkcja w zaleznosci od parametru kind_of_conn laczy sie, lub rozlacza 
  z baza danych. Dane odnosnie polaczenia pobierane sa z pliku zdefiniowanego w makrze DB_CONF.
  @param kind_of_conn  - Rodzaj polaczenia. Parametr moze miec wartosc 0 lub 1.
  Kiedy kind_of_conn ma wartosc 0, wtedy nastepuje polaczenie z baza, kiedy 1 wtedy polaczenie zostaje zerwane. 
  @return kind_of_result  - Funkcja zwraca wartosc 0, gdy zadanie zostalo wykonane poprawnie, -1 gdy nastapil blad.
*/ 
int xdfConnSQL (xdfConnModule *conn_module);

struct xdfDialogMsgStruct
{
  GString *title;
  GString *msg;
  GtkMessageType type;
};


void xdfDisconnect (xdfConnModule *conn_mod);

int xdfLogBehaviour (char *msg);

GtkWidget* xdfDialogMsg (struct xdfDialogMsgStruct *strDialog);

#endif /* __CALLBACKS_H__ */
