/*
 * moduleNumbers.c
 *
 * Modul do programu xdefactor-ng.
 * Autor: Przemysaw Sowa <przemos@e-demokracja.pl>
 * 
 * Modul ten wyswietla okienko pozwalajace sprawdzic poprawnosc
 * numberow: NIP, REGON i PESEL.
 *
 * 31.01.2003
 */
#include <string.h>
#include <ctype.h>
#include "moduleNumbers.h"

GtkWidget *window1;
GtkWidget *entry1;
GtkWidget *entry2;
GtkWidget *entry3;

int check_nip( const char *NIP )
{
	char number[11];
	int i, j, sum;
	i = j = number[10] = 0;
	
	//  Poprawny number NIP ma 13 cyfr (wraz z kreskami)
	if (strlen(NIP) != 13)
		return (FALSE);
	
	// Pozbywamy sie kresek i ew. innych literowych smieci
    // (gdyby ktos wpisal pomylkowo litere)
	for (i = 0; i < 13; i++) {
		if (isdigit(NIP[i])) {
			number[j] = NIP[i];
			j++;
		}
	}

	// Sprawdzmy, czy mamy poprawna liczbe cyfr
  if (strlen(number) != 10)
		return (FALSE);

	// Mnozymy poszczegolne cyfry numberu przez wspolczynniki wagowe
  sum = (number[0] - 48) * 6;
  sum += (number[1] - 48) * 5;
	sum += (number[2] - 48) * 7;
	sum += (number[3] - 48) * 2;
	sum += (number[4] - 48) * 3;
	sum += (number[5] - 48) * 4;
	sum += (number[6] - 48) * 5;
	sum += (number[7] - 48) * 6;
	sum += (number[8] - 48) * 7;
	
    // Otrzymany wynik dzielimy modulo przez 11
  sum  = sum % 11;
    
	// Sprawdzamy z sum kontrolna wpisanego numberu
  if (sum != number[9] - 48)
		return (FALSE);
	
	return (TRUE);
}

int check_pesel( const char *PESEL )
{
	int i = 0, sum;
	
	//  Poprawny number PESEL ma 11 cyfr
	if (strlen(PESEL) != 11)
		return (FALSE);
	
	// Poprawny PESEL nie moze zawierac liter
	for (i = 0; i < 11; i++) {
		if (!isdigit(PESEL[i]))
			return (FALSE);
	}

	// Mnozymy poszczegolne cyfry numberu przez wspolczynniki wagowe
  sum = (PESEL[0] - 48) * 1;
  sum += (PESEL[1] - 48) * 3;
	sum += (PESEL[2] - 48) * 7;
	sum += (PESEL[3] - 48) * 9;
	sum += (PESEL[4] - 48) * 1;
	sum += (PESEL[5] - 48) * 3;
	sum += (PESEL[6] - 48) * 7;
	sum += (PESEL[7] - 48) * 9;
	sum += (PESEL[8] - 48) * 1;
	sum += (PESEL[9] - 48) * 3;
	
  // Otrzymany wynik dzielimy modulo przez 11 i odejmujemy od 10
  sum  = 10 - (sum % 10);
    
	// Jezeli wynik = 10, wtedy sum kontrolna = 0
	if (sum == 10)
		sum = 0;

	// Sprawdzamy z sum kontrolna wpisanego numberu
  if (sum != PESEL[10] - 48)
		return (FALSE);
	
	return (TRUE);
}

int check_regon( const char *REGON )
{
	int i = 0, sum;
	
	//  Poprawny number REGON ma 9 cyfr
	if (strlen(REGON) != 9)
		return (FALSE);
	
	// Poprawny REGON nie moze zawierac liter
	for (i = 0; i < 9; i++) {
		if (!isdigit(REGON[i]))
			return (FALSE);
	}

	// Mnozymy poszczegolne cyfry numberu przez wspolczynniki wagowe
  sum = (REGON[0] - 48) * 8;
  sum += (REGON[1] - 48) * 9;
	sum += (REGON[2] - 48) * 2;
	sum += (REGON[3] - 48) * 3;
	sum += (REGON[4] - 48) * 4;
	sum += (REGON[5] - 48) * 5;
	sum += (REGON[6] - 48) * 6;
	sum += (REGON[7] - 48) * 7;
	
	
  // Otrzymany wynik dzielimy modulo przez 11 i odejmujemy od 10
  sum = sum % 11;
    
	// Jezeli wynik = 10, wtedy sum kontrolna = 0
	if (sum == 10)
		sum = 0;

	// Sprawdzamy z sum kontrolna wpisanego numberu
  if (sum != REGON[8] - 48)
  	return (FALSE);
	
	return (TRUE);
}

void check_clicked( GtkWidget *widget,
            gpointer   data )
{
	GtkWidget *dialog;
	gchar *message;
	GtkMessageType msg_type;
	
	// Sprawdzamy NIP
  if (strlen(gtk_entry_get_text(GTK_ENTRY(entry1)))) {
		if (!check_nip(gtk_entry_get_text(GTK_ENTRY(data)))) {
			message = "Niepoprawny numer NIP";
			msg_type = GTK_MESSAGE_ERROR;
		} else {
			message = "Ten numer NIP jest poprawny";
			msg_type = GTK_MESSAGE_INFO;
		}
		
		dialog = gtk_message_dialog_new (GTK_WINDOW(window1),
									  GTK_DIALOG_DESTROY_WITH_PARENT,
									  msg_type,
									  GTK_BUTTONS_CLOSE,
									  message);
		gtk_dialog_run (GTK_DIALOG (dialog));
		gtk_widget_destroy (dialog);
	}

	// Sprawdzamy REGON
  if ( strlen(gtk_entry_get_text(GTK_ENTRY(entry2)))) {
		if (!check_regon(gtk_entry_get_text(GTK_ENTRY(entry2)))) {
			message = "Niepoprawny numer REGON";
			msg_type = GTK_MESSAGE_ERROR;
		} else {
			message = "Ten numer REGON jest poprawny";
			msg_type = GTK_MESSAGE_INFO;
		}
		
		dialog = gtk_message_dialog_new (GTK_WINDOW(window1),
									  GTK_DIALOG_DESTROY_WITH_PARENT,
									  msg_type,
									  GTK_BUTTONS_CLOSE,
									  message);
		gtk_dialog_run (GTK_DIALOG (dialog));
		gtk_widget_destroy (dialog);
	}
	
	// Sprawdzamy PESEL
  if ( strlen(gtk_entry_get_text(GTK_ENTRY(entry3)))) {
		if (!check_pesel(gtk_entry_get_text(GTK_ENTRY(entry3)))) {
			message = "Niepoprawny numer PESEL";
			msg_type = GTK_MESSAGE_ERROR;
		} else {
			message = "Ten numer PESEL jest poprawny";
			msg_type = GTK_MESSAGE_INFO;
		}
		
		dialog = gtk_message_dialog_new (GTK_WINDOW(window1),
									  GTK_DIALOG_DESTROY_WITH_PARENT,
									  msg_type,
									  GTK_BUTTONS_CLOSE,
									  message);
		gtk_dialog_run (GTK_DIALOG (dialog));
		gtk_widget_destroy (dialog);
	}

}

void end_clicked( GtkWidget *widget,
            gpointer   data )
{
	gtk_widget_destroy(GTK_WIDGET(data));
}

void moduleNumbers ( )
{
  GtkWidget *fixed1;
  GtkWidget *button1;
  GtkWidget *alignment1;
  GtkWidget *hbox1;
  GtkWidget *image1;
  GtkWidget *label2;
  GtkWidget *button2;
  GtkWidget *label4;
  GtkWidget *label3;
  GtkWidget *label1;

  window1 = gtk_window_new (GTK_WINDOW_TOPLEVEL);
  gtk_widget_set_usize (window1, 210, 150);
  gtk_window_set_title (GTK_WINDOW (window1), "Numery ewidencyjne");
  gtk_window_set_resizable (GTK_WINDOW (window1), FALSE);
  gtk_window_set_position (GTK_WINDOW (window1), GTK_WIN_POS_CENTER);
	
  fixed1 = gtk_fixed_new ();
  gtk_widget_show (fixed1);
  gtk_container_add (GTK_CONTAINER (window1), fixed1);

  entry1 = gtk_entry_new ();
  gtk_widget_show (entry1);
  gtk_fixed_put (GTK_FIXED (fixed1), entry1, 72, 8);
  gtk_widget_set_usize (entry1, 126, 24);

  button1 = gtk_button_new ();
  gtk_widget_show (button1);
  gtk_fixed_put (GTK_FIXED (fixed1), button1, 8, 112);
  gtk_widget_set_usize (button1, 96, 32);

  alignment1 = gtk_alignment_new (0.5, 0.5, 0, 0);
  gtk_widget_show (alignment1);
  gtk_container_add (GTK_CONTAINER (button1), alignment1);

  hbox1 = gtk_hbox_new (FALSE, 2);
  gtk_widget_show (hbox1);
  gtk_container_add (GTK_CONTAINER (alignment1), hbox1);

  image1 = gtk_image_new_from_stock ("gtk-apply", GTK_ICON_SIZE_BUTTON);
  gtk_widget_show (image1);
  gtk_box_pack_start (GTK_BOX (hbox1), image1, FALSE, FALSE, 0);

  label2 = gtk_label_new_with_mnemonic ("_Sprawd\305\272");
  gtk_widget_show (label2);
  gtk_box_pack_start (GTK_BOX (hbox1), label2, FALSE, FALSE, 0);
  gtk_label_set_justify (GTK_LABEL (label2), GTK_JUSTIFY_LEFT);

  entry2 = gtk_entry_new ();
  gtk_widget_show (entry2);
  gtk_fixed_put (GTK_FIXED (fixed1), entry2, 72, 40);
  gtk_widget_set_usize (entry2, 126, 24);

  entry3 = gtk_entry_new ();
  gtk_widget_show (entry3);
  gtk_fixed_put (GTK_FIXED (fixed1), entry3, 72, 72);
  gtk_widget_set_usize (entry3, 126, 24);

  button2 = gtk_button_new_from_stock ("gtk-close");
  gtk_widget_show (button2);
  gtk_fixed_put (GTK_FIXED (fixed1), button2, 104, 112);
  gtk_widget_set_usize (button2, 96, 32);

  label4 = gtk_label_new ("PESEL:");
  gtk_widget_show (label4);
  gtk_fixed_put (GTK_FIXED (fixed1), label4, 8, 76);
  gtk_widget_set_usize (label4, 48, 16);
  gtk_label_set_justify (GTK_LABEL (label4), GTK_JUSTIFY_LEFT);

  label3 = gtk_label_new ("REGON:");
  gtk_widget_show (label3);
  gtk_fixed_put (GTK_FIXED (fixed1), label3, 8, 44);
  gtk_widget_set_usize (label3, 56, 16);
  gtk_label_set_justify (GTK_LABEL (label3), GTK_JUSTIFY_LEFT);

  label1 = gtk_label_new ("NIP:");
  gtk_widget_show (label1);
  gtk_fixed_put (GTK_FIXED (fixed1), label1, 8, 12);
  gtk_widget_set_usize (label1, 34, 16);
  gtk_label_set_justify (GTK_LABEL (label1), GTK_JUSTIFY_LEFT);

  g_signal_connect (G_OBJECT (button1), "clicked",
		      G_CALLBACK (check_clicked), entry1); 
  
  gtk_signal_connect (GTK_OBJECT (button2), "clicked",
                      G_CALLBACK (end_clicked),
                      GTK_OBJECT (window1));
  gtk_widget_show (window1);					  
}
