#include <gtk/gtk.h>
#include <stdio.h>
#include <errno.h>
#include <string.h>
#include <gmodule.h>
#include "support.h"

#define MODULES_CONF "/etc/defactor-ng/x/modules.conf"
#define MODULES_DESC_PATH "/etc/defactor-ng/x"
#define TAG_AUTHORS "author="
#define TAG_NAME "name="
#define TAG_MENU "menu="
#define TAG_SUBMENU "submenu="
#define TAG_DESC "about_module="

typedef struct _PluginInfo PluginInfo;

struct _PluginInfo {
	GModule *handle;
	gchar *desc;
	gchar *name;
	gchar *authors;
	gchar *menu;
	gchar *file;
	gchar *desc_file;
};

/* Kolumny listy modulow. */
enum
{
	CHECKED_COLUMN,
	NAME_COLUMN,
	INFO_COLUMN,
  	N_COLUMNS
};

static void
plugin_manager_update_info (GtkWidget *dialog, PluginInfo *info)
{
	gchar *filename;
	gchar *author;
	gchar *name;
	gchar *menu;
	
	filename = g_strdup_printf ("%s: %s", _("Plik z modu\305\202em"), info->file);
	author = g_strdup_printf ("%s: %s", _("Autorzy"), info->authors);
	name = g_strdup_printf ("%s: %s", _("Wtyczka"), info->name);
	menu = g_strdup_printf ("%s: %s", _("Pozycja w menu"), info->menu);

	gtk_label_set_text (GTK_LABEL (lookup_widget(dialog, "desc_label")), info->desc);
	gtk_label_set_text (GTK_LABEL (lookup_widget(dialog, "author_label")), author);
	gtk_label_set_text (GTK_LABEL (lookup_widget(dialog, "file_label")), filename);
	gtk_label_set_text (GTK_LABEL (lookup_widget(dialog, "menu_label")), menu);
	gtk_label_set_text (GTK_LABEL (lookup_widget(dialog, "name_label")), name);

	g_free (filename);
	g_free (author);
	g_free (name);
	g_free (menu);
}

static PluginInfo *
plugin_manager_get_selected_plugin (GtkTreeView  *tree)
{
	PluginInfo *info = NULL;
	GtkTreeModel *model;
	GtkTreeIter iter;
	GtkTreeSelection *selection;

	model = gtk_tree_view_get_model (tree);
	g_return_val_if_fail (model != NULL, NULL);

	selection = gtk_tree_view_get_selection (tree);
	g_return_val_if_fail (selection != NULL, NULL);

	if (gtk_tree_selection_get_selected (selection, NULL, &iter))
	{
		gtk_tree_model_get (model, &iter, INFO_COLUMN, &info, -1);
	}
	
	return info;
}

static void
plugin_manager_set_active (GtkTreeIter *iter, GtkTreeModel *model, gboolean active)
{
	gtk_list_store_set (GTK_LIST_STORE (model), iter, CHECKED_COLUMN,
						active, -1);
}

static void
plugin_manager_toggle_all (GtkTreeView *tree)
{
	GtkTreeModel *model;
	GtkTreeIter iter;
	static gboolean active;


	active ^= 1;

	model = gtk_tree_view_get_model (tree);

	g_return_if_fail (model != NULL);

	gtk_tree_model_get_iter_root (model, &iter);

	do {
		plugin_manager_set_active (&iter, model, active);		
	}
	while (gtk_tree_model_iter_next (model, &iter));
}

static void
column_clicked_cb (GtkTreeViewColumn *tree_column, gpointer data)
{
	GtkTreeView *tree = data;


	g_return_if_fail (tree != NULL);

	plugin_manager_toggle_all (tree);
}

static void
cursor_changed_cb (GtkTreeView  *view, gpointer data)
{
	GtkWidget *dialog = data;
	PluginInfo *info;


	info = plugin_manager_get_selected_plugin (view);
	g_return_if_fail (info != NULL);

	plugin_manager_update_info (dialog, info);
}

static void
active_toggled_cb (GtkCellRendererToggle     *cell,
		   gchar                 *path_str,
		   gpointer               data)
{
	GtkWidget *tree = GTK_WIDGET(data);
	GtkTreeIter  iter;
	GtkTreePath *path = gtk_tree_path_new_from_string (path_str);
	GtkTreeModel *model;
	gboolean active;

	model = gtk_tree_view_get_model (GTK_TREE_VIEW (tree) );
	g_return_if_fail (model != NULL);

	/* get toggled iter */
	gtk_tree_model_get_iter (model, &iter, path);

	if (&iter != NULL) {
		gtk_tree_model_get (model, &iter, CHECKED_COLUMN, &active, -1);
		plugin_manager_set_active (&iter, model, !active);
	}
  
	/* clean up */
	gtk_tree_path_free (path);
}

static void
row_activated_cb (GtkTreeView *tree_view,
		  GtkTreePath *path,
		  GtkTreeViewColumn *column,
		  gpointer data)
{
	GtkTreeIter iter;
	GtkTreeModel *model;
	gboolean active;
	
	model = gtk_tree_view_get_model (tree_view);

	g_return_if_fail (model != NULL);
	
	gtk_tree_model_get_iter (model, &iter, path);

	g_return_if_fail (&iter != NULL);
	
	if (&iter != NULL) {
		gtk_tree_model_get (model, &iter, CHECKED_COLUMN, &active, -1);
		plugin_manager_set_active (&iter, model, !active);
	}

}

/*
 * Funkcja: free_plugin_info
 * 
 * Zwraca zasoby zajete przez zmienne tekstowe struktury.
 * 
 * UWAGA:
 * Zmienna "handle" type GModule struktury pozostaje nietknieta,
 * modul, na ktory wskazuje nie jest zwalniany.
 */
void
free_plugin_info(PluginInfo *info)
{
	g_free(info->desc);
	g_free(info->name);
	g_free(info->authors);
	g_free(info->menu);
	g_free(info->file);
	g_free(info->desc_file);
	g_free(info);
}

/*
 * Funkcja: read_plugin_desc_file
 * 
 * Wczytuje plik opisujacy modul (z /etc/defactor-ng/x/modules/Modul.conf)
 * i zwraca wskaznik do struktury PluginInfo opisujacej modul.
 * 
 * UWAGA:
 * Zwrocona struktura powinna zostac zwolniona za pomoca free_plugin_info,
 * kiedy nie potrzebna.
 */

PluginInfo *
read_plugin_desc_file(gchar *desc_file)
{
	PluginInfo *info;
	FILE *conf;
	gchar buffer[255], *menu = NULL;
	
	conf = fopen(desc_file, "r");
	if (!conf) {
		return (NULL);
	}

	info = g_new0(PluginInfo, 1);

	/* Przetwarzanie pliku opsiujacego modul */	
	while ( fgets(buffer, 255, conf) != NULL ) {
//		g_print("Lina: %s", buffer);
		
		if (buffer[0] != '#') {
			buffer[strlen(buffer)-1] = '\0';

			if (strstr(buffer, TAG_NAME) == buffer) {
				info->name = g_strdup(buffer + strlen(TAG_NAME));
			}
			if (strstr(buffer, TAG_AUTHORS) == buffer) {
				info->authors = g_strdup(buffer + strlen(TAG_AUTHORS));
			}
			if (strstr(buffer, TAG_MENU) == buffer) {
				menu = g_strdup(buffer + strlen(TAG_MENU));
			}
			if (strstr(buffer, TAG_SUBMENU) == buffer) {
				info->menu = g_strdup_printf("%s/%s", menu, buffer + strlen(TAG_SUBMENU));
				g_free(menu);
			}
			if (strstr(buffer, TAG_DESC) == buffer) {
				info->desc = g_strdup(buffer + strlen(TAG_DESC));
			}			
		}			
	}
	
	fclose (conf);				
	return info;
}
	

gchar *
read_modules_conf(GtkListStore *store)
{
	PluginInfo *info;
	GtkTreeIter iter;
	FILE *modules;
	gchar *desc_file, *plugin_file;
	gchar buffer[255];
	gboolean active;
	
	if (!g_file_test (MODULES_CONF, G_FILE_TEST_EXISTS)) {
		g_print("Plik %s nie istnieje!\n", MODULES_CONF);
		return (NULL);
	}
	
	modules = fopen(MODULES_CONF, "r");
	if (!modules) {
		g_print("%s: %s\n", MODULES_CONF, strerror(errno));
		return (NULL);
	}
	
	while ( fgets(buffer, 255, modules) != NULL ) {
//		g_print("Lina: %s", buffer);

		buffer[strlen(buffer)-1] = '\0';
		
		// Czy mamy wykomentowany modul?
		if (strstr(buffer, "#/modules/") == buffer) {
			
			// Usun '#' z nazwy pliku opisowego
			desc_file = g_strdup(buffer+1);
			// Wczytaj nazwe pliku modulu
			fgets(buffer, 255, modules);
			buffer[strlen(buffer)-1] = '\0';
			// Usun '#' z nazwy pliku modulu
			plugin_file = g_strdup(buffer+1);
			active = FALSE;

		} else if (buffer[0] != '#' ) {
			// Mamy aktywny modul		
			desc_file = g_strdup(buffer);
			
			fgets(buffer, 255, modules);
			buffer[strlen(buffer)-1] = '\0';
			plugin_file = g_strdup(buffer);
			active = TRUE;
		}
			
		buffer[0] = '\0';
		strcat(buffer, MODULES_DESC_PATH);
		strcat(buffer, desc_file);
		
		info = read_plugin_desc_file(buffer);
		if (info != NULL) {
			info->desc_file = desc_file;			
			info->file = plugin_file;			
			gtk_list_store_append (store, &iter);
			gtk_list_store_set (store, &iter,
								CHECKED_COLUMN, active,
								NAME_COLUMN, info->name,
								INFO_COLUMN, info,
								-1);
		}
	}
		
	fclose(modules);
	return (NULL);
}

void
load_plugins_list(GtkListStore *store)
{
	read_modules_conf(store);
}

void
construct_list(GtkWidget *tree, GtkWidget *main_window)
{
	GtkListStore *store;
	GtkCellRenderer *renderer;
	GtkTreeViewColumn *column;
	
	store = gtk_list_store_new (N_COLUMNS,
								G_TYPE_BOOLEAN,
								G_TYPE_STRING,
								G_TYPE_POINTER);


	load_plugins_list (store);
	
	gtk_tree_view_set_model (GTK_TREE_VIEW(tree), GTK_TREE_MODEL (store));	

	renderer = gtk_cell_renderer_toggle_new ();
	column = gtk_tree_view_column_new_with_attributes ("Wczytaj",
                                                   		renderer,
                                                   		"active", CHECKED_COLUMN,
                                                   		NULL);
	gtk_tree_view_column_set_clickable (column, TRUE);
	g_signal_connect (G_OBJECT (column), "clicked", 
					 G_CALLBACK (column_clicked_cb), tree);
					 
	gtk_tree_view_append_column (GTK_TREE_VIEW (tree), column);	
	g_signal_connect (G_OBJECT (renderer), "toggled",
					  G_CALLBACK (active_toggled_cb), tree);
	
	renderer = gtk_cell_renderer_text_new ();
	column = gtk_tree_view_column_new_with_attributes ("Wtyczka",
                                                   		renderer,
                                                   		"text", NAME_COLUMN,
                                                   		NULL);
	gtk_tree_view_append_column (GTK_TREE_VIEW (tree), column);	
	
	g_signal_connect (G_OBJECT (tree), "row_activated",
					G_CALLBACK (row_activated_cb), NULL);

	g_signal_connect (G_OBJECT (tree), "cursor-changed",
					G_CALLBACK (cursor_changed_cb), main_window);

}

void
plugin_manager_save_config(GtkTreeView *tree)
{
	GtkTreeModel *model;
	GtkTreeIter iter;
	static gboolean active;
	FILE *conf;
	PluginInfo *info;
	static gchar *module;
	
	model = gtk_tree_view_get_model (tree);
	g_return_if_fail (model != NULL);

	gtk_tree_model_get_iter_root (model, &iter);
	
	conf = fopen(MODULES_CONF, "w");
	if (!conf) {
		g_print("%s: %s\n", MODULES_CONF, strerror(errno));
		return;
	}

	
	do {
		gtk_tree_model_get (model, &iter, CHECKED_COLUMN, &active, -1);
		gtk_tree_model_get (model, &iter, INFO_COLUMN, &info, -1);
		
		if (!active) {
			module = g_strdup_printf("#%s\n#%s\n", info->desc_file,	info->file);
		} else {
			module = g_strdup_printf("%s\n%s\n", info->desc_file,	info->file);
		}
		fputs(module, conf);
		g_free(module);
	}
	while (gtk_tree_model_iter_next (model, &iter));
	
	fclose(conf);
}

void
plugin_manager_cleanup(GtkTreeView *tree)
{
	GtkTreeModel *model;
	GtkTreeIter iter;
	PluginInfo *info;
	
	model = gtk_tree_view_get_model (tree);
	g_return_if_fail (model != NULL);

	gtk_tree_model_get_iter_root (model, &iter);
	
	do {
		gtk_tree_model_get (model, &iter, INFO_COLUMN, &info, -1);
		free_plugin_info(info);
	}
	while (gtk_tree_model_iter_next (model, &iter));

}
